
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ConjGradDir.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [point,step,par,info]= ...
%                     ConjGradDir(fun,point,step,tune,par,info);
% construct the conjugate gradient direction

function [point,step,par,info]=ConjGradDir(fun,point,step,par,tune,info)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%% get gamma %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[point,step,par,info]=getGam(fun,point,step,par,tune,info);

% find subspace direction
if point.m0>0,

    % find ingredients for projected subspace minimizer
    h=point.S(:,par.hist)'*point.g;
    % do the following only if h is significant                   
    Hoh=point.H(par.hist,par.hist);
    step.q=(step.p'*point.Y(:,par.hist))';
    rhs=[-h step.q];
    warning off
    sol=Hoh\rhs;
    warning on 
    
    ok = (any(isnan(sol))|any(isinf(sol)));
    if ~ok
       z=sol(:,1);
       step.r=sol(:,2);
    
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%% get denom %%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        [par]      = regDenom(point,step,par,tune);
        zeta       = (step.gTp+step.q'*z)/par.denom;
        okz        = isnan(zeta);
        if okz, zeta = tune.zetamax; end
        zeta       = min(tune.zetamax,max(tune.zetamin,zeta));
        z          = z+zeta*step.r;
        par.cosine = -step.gTp*zeta+h'*z; % finite only if z, zeta finite
        step.p     = -step.p*zeta+point.S(:,par.hist)*z;
        
        
        clear sol rhs denom z zeta 
        
       


        if info.prt>=1, 
            disp(['ConjGradDir: the subspace direction has been',...
             ' computed (m0=',num2str(point.m0),')']); 
        end
    else
        % no subspace step possible
        % here doing instead an ''exact'' line search saves function values
        zeta        = step.gTp/par.gamma;
        okz        = isnan(zeta);
        if okz, zeta = tune.zetamax; end
        zeta        = min(tune.zetamax,max(tune.zetamin,zeta));
        par.cosine  = -step.gTp*zeta; % is finite only if z is finite 
        step.p      = -step.p*zeta;
        if info.prt>=1, 
            disp(['ConjGradDir: no subspace step possible;',...
                 ' m0=',num2str(point.m0)]); 
        end
        
        
        clear zeta 
        
    end

else
    % no subspace step possible
    % here doing instead an ''exact'' line search saves function values
     
     zeta       = step.gTp/par.gamma;
     okz        = isnan(zeta);
     if okz, zeta = tune.zetamax; end
     zeta       = min(tune.zetamax,max(tune.zetamin,zeta));
     
     par.cosine  = -step.gTp*zeta; % is finite only if z is finite 
     step.p      = -step.p*zeta;
     
     
         
     if info.prt>=1, 
            disp(['ConjGradDir: no subspace step possible;',...
                 ' m0=',num2str(point.m0)]); 
      end
     clear zeta 
     
end;

point.xnew=max(point.low,min(point.x+step.p,point.upp));

step.p=point.xnew-point.x;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   