%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%% enforceAngle.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [step]=enforceAngle(point,step,par,tune)
% enforce the angle condition
%
% for details of input and output structures see lmbopt.m
%


function [step]=enforceAngle(point,step,par,tune)

if ~par.firstAngle, step.pI=step.p(point.I); end

sigma=point.gI'*step.pI;
if sigma>0,
  % move away from maximizer or saddle point
  step.pI=-step.pI; sigma=-sigma; step.gTp = step.pI'*point.gI;
end;
sigma1  = point.gI'*point.gI; sigma2  = step.pI'*step.pI; 
sigma12 = sigma1*sigma2;
sigmanew = sigma/sqrt(sigma12);
if sigmanew<=-tune.Deltaangle,
else
  w=(sigma12*max(tune.Deltaw,1-sigmanew^2))/(1-tune.Deltaangle^2);
  t=(sigma+tune.Deltaangle*sqrt(w))/sigma1;
  if w>0 & isfinite(t), step.pI=step.pI-t*point.gI; 
  else 
    % this includes the case where pI=0  
    if par.firstAngle
       step.lamBB  = max(point.Lam);
       step.lamBB  = min(tune.lammax,max(tune.lammin,step.lamBB)); 
       step.pI     = -step.lamBB*point.gI; 
    else
       step.pI     = -point.gI;
    end
  end
  step.gTp = step.pI'*point.gI;
end % of if sigmanew

clear sigma sigma1 sigma2 sigma12 t w sigmanew


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   