%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% funfg.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compute function value at x and its gradient

function [f,g] = funfg(x,prob);
cas=prob.cas;
switch cas
    case 1
       A=prob.A; b=prob.b; r=A*x-b; f=r'*r/2; g=(r'*A)';
    case 2 % Chained wood
        % compute the function
        n=prob.n; f=0;
        for i=2:2:n-2
            A=x(i-1)^2-x(i);
            B=x(i-1)-1;
            C=x(i+1)^2-x(i+2);
            D=x(i+1)-1;
            U=x(i)+x(i+2)-2;
            V=x(i)-x(i+2);
            f=f+100*A^2+B^2+90*C^2+D^2+10*U^2+0.1*V^2;
        end 
        % compute the gradient
        for i=1:n, g(i)=0; end
        for i=2:2:n-2
            A       = x(i-1)^2-x(i);
            B       = x(i-1)-1;
            C       = x(i+1)^2-x(i+2);
            D       = x(i+1)-1;
            U       = x(i)+x(i+2)-2;
            V       = x(i)-x(i+2);
            g(i-1) = g(i-1)+400*x(i-1)*A+2*B;
            g(i)   = g(i)-200*A+20*U+0.2*V;
            g(i+1) = g(i+1)+360*x(i+1)*C+2*D;
            g(i+2) = g(i+2)-180*C+20*U-0.2*V;
        end    
        g=g(:);
    case 3  %  Brent
        % compute the function
        n=prob.n; f=0;
        for j=1:n
            if j==1
              FA=3*x(j)*(x(j+1)-2*x(j))+0.25*x(j+1)^2;
            elseif j==n
              FA=3*x(j)*(20-2*x(j)+x(j-1))+ 0.25*(20-x(j-1))^2;
            else
              FA=3*x(j)*(x(j+1)-2*x(j)+x(j-1))+ 0.25*(x(j+1)-x(j-1))^2;
            end
            f=f+FA^2;
        end  
        f=0.5*f;
        % compute the gradient
        for i=1:n, g(i)=0; end
        for j=1:n
            if j==1
              FA=3*x(j)*(x(j+1)-2*x(j))+0.25*x(j+1)^2;
              g(j)=g(j)+3*(x(j+1)-4*x(j))*FA;
              g(j+1)=g(j+1)+(3*x(j)+0.5*x(j+1))*FA;
            elseif j==n
              FA=3*x(j)*(20-2*x(j)+x(j-1))+ 0.25*(20-x(j-1))^2;
              g(j-1)=g(j-1)+(3*x(j)-0.5*(20-x(j-1)))*FA;
              g(j)=g(j)+3*(20-4*x(j)+x(j-1))*FA;
            else
              FA=3*x(j)*(x(j+1)-2*x(j)+x(j-1))+ 0.25*(x(j+1)-x(j-1))^2;
              g(j-1)=g(j-1)+(3*x(j)-0.5*(x(j+1)-x(j-1)))*FA;
              g(j)=g(j)+3*(x(j+1)-4.0*x(j)+x(j-1))*FA;
              g(j+1)=g(j+1)+(3*x(j)+0.5*(x(j+1)-x(j-1)))*FA;
            end
        end  
        g=g(:);
    case 4 % Variational 1
        % compute the function
        n=prob.n; P=1/(n+1); f=0;
        for j=1:n
           if j==1
              f=f+0.25*x(j)^2/P+0.125*x(j+1)^2/P+P*(exp(x(j))-1) ;
           elseif j==n 
              f=f+0.25*x(j)^2/P+0.125*x(j-1)^2/P+P*(exp(x(j))-1);
           else
              f=f+0.125*(x(j+1)-x(j-1))^2/P+P*(exp(x(j))-1) ;
           end   
        end 
       % compute the gradient
       for i=1:n, g(i)=0; end
       for j=1:n
       if j==1
          g(j)=g(j)+0.5*x(j)/P+P*exp(x(j));
          g(j+1)=g(j+1)+0.25*x(j+1)/P;
       elseif j==n 
          g(j)=g(j)+0.5*x(j)/P+P*exp(x(j));
          g(j-1)=g(j-1)+0.25*x(j-1)/P;
       else
          A = 0.25*(x(j+1)-x(j-1))/P;
          g(j)  =g(j)+P*exp(x(j));
          g(j-1)=g(j-1)-A;
          g(j+1)=g(j+1)+A;
       end   
       end
       g=g(:);
    
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%